#!/bin/bash

version_gt() { test "$(echo "$@" | tr " " "\n" | sort -V | tail -n 1)" == "$1"; }

check_exists() { if ! hash $1 2>/dev/null ; then echo $1 not found ; exit 1 ; fi ; }

check_exists dd
check_exists sfdisk
check_exists mkfs.ext4
check_exists lsblk
check_exists id
check_exists mkfs.f2fs
check_exists lsblk


if [ "$#" -ne 3 ] ; then 
	echo "Usage:   Write to device : $0 rootfs.tgz u-boot /dev/mmcblk1"
	echo "       Create image file : $0 rootfs.tgz u-boot rootfs.img"
	exit 1
fi

if [ $(id -u) -ne 0 ] ; then echo "Must be run as root"; exit 0; fi

UBOOT=$2
IMAGE=$1
DISK=$3

echo Image:$IMAGE Disk:$DISK U-Boot:$UBOOT

if [ ! -f "${UBOOT}/MLO" ] || [ ! -f "${UBOOT}/u-boot.img" ] ; then echo "u-boot files not found"; exit 1; fi

if [ ! -f $IMAGE ] ; then echo "rootfs image $IMAGE not found."; exit 1; fi

if [[ $DISK != /dev/* ]] ; then
    echo Creating disk image
    DISKIMAGE=$DISK
    dd if=/dev/zero bs=1 count=0 seek=8G of="$DISKIMAGE"
    DISK=$(losetup -f -P --show "${DISKIMAGE}")
else
    lsblk $DISK
    #if [[ $? != 0 ]] ; then echo Failed; exit 1; fi

    echo "Installing to $DISK , make sure all partitions are unmounted"
    if [[ "$INITHWTEST" -ne 1 ]] ; then
	    read -p "Are you sure you want to continue? <y/N> " prompt
	    if ! [[ $prompt =~ [yY](es)* ]] ; then echo "Exiting.." ; exit 1 ; fi
    fi
fi
#exit 0

if [[ $DISK == /dev/mmcblk* ]] || [[ $DISK == /dev/loop* ]] ; then
	PART=${DISK}p2
    BOOTPART=${DISK}p1

else
	PART=${DISK}2
    BOOTPART=${DISK}1
fi

echo $PART $BOOTPART
#exit

echo "Clearing partition table"
dd if=/dev/zero of=${DISK} bs=1M count=10
if [[ $? != 0 ]] ; then echo Partitioning failed.; exit 1; fi

echo "Installing U-Boot"
dd if="$UBOOT/MLO" of=${DISK} count=1 seek=1 bs=128k
dd if="$UBOOT/u-boot.img" of=${DISK} count=2 seek=1 bs=384k

echo "Creating partition"

SFVER=$(sfdisk -v | awk '{print $4}')
echo Found sfdisk version: ${SFVER}
if version_gt $SFVER 2.26.0; then
echo "> 2.26.0"
sfdisk ${DISK} <<-__EOF__
1M,256M,0x83,*
,,0x83,
__EOF__
else
echo "< 2.26.0"
sfdisk --in-order --Linux --unit M ${DISK} <<-__EOF__
1,,0x83,*
,,0x83,
__EOF__
fi

if [[ $? != 0 ]] ; then echo Partitioning failed.; exit 1; fi

echo "Formatting partitions"
#mkfs.ext4 -O ^64bit ${PART} -L rootfs
mkfs.ext4 -O ^64bit ${BOOTPART} -L boot

# older mkfs.f2fs versions don't have -f, newer versions need -f if fs exists
# older versions don't report version either, so assume if -V fails, -f isn't there

mkfs.f2fs -V > /dev/null 2>/dev/null
if [[ $? == 0 ]] ; then
    mkfs.f2fs ${PART} -f -l rootfs
else
    mkfs.f2fs ${PART} -l rootfs
fi


if [[ $? != 0 ]] ; then echo Formatting failed.; exit 1; fi

mkdir rootfs
mount ${PART} rootfs
mkdir rootfs/boot
mount ${BOOTPART} rootfs/boot

if [[ $? != 0 ]] ; then echo Mounting ${PART} failed.; exit 1; fi


echo "Installing rootfs"

if hash bar 2>/dev/null ; then
	bar $IMAGE | tar --numeric-owner -xzf - -C rootfs
else
	echo Install "bar" for progress bar
	tar --numeric-owner -xzf $IMAGE -C rootfs
fi

ROOTPARTUUID=$(blkid -p $PART -s PART_ENTRY_UUID -o value)
BOOTPARTUUID=$(blkid -p $BOOTPART -s PART_ENTRY_UUID -o value)

#ROOTPARTUUID=$(lsblk -n -o PARTUUID $PART)
#BOOTPARTUUID=$(lsblk -n -o PARTUUID $BOOTPART)
# fs uuid sometimes failed, use PARTUUID for everything
#ROOTFSUUID=$(lsblk -n -o UUID $PART)
#BOOTFSUUID=$(lsblk -n -o UUID $BOOTPART)


sed -i -e 's/linux \/boot/linux /g' rootfs/boot/extlinux/linux.cfg
sed -i -e 's/initrd \/boot/initrd /g' rootfs/boot/extlinux/linux.cfg
sed -i -e "s/append root=\/dev\/mmcblk0p1/append root=PARTUUID=$ROOTPARTUUID/g" rootfs/boot/extlinux/linux.cfg
sed -i -e "s/EXTLINUX_ROOT=\"root=\/dev\/mmcblk0p1\"/EXTLINUX_ROOT=\"root=PARTUUID=$ROOTPARTUUID\"/g" rootfs/etc/default/pyra-extlinux

cat > rootfs/etc/fstab << EOF
# /etc/fstab: static file system information.
#
# Use 'blkid' to print the universally unique identifier for a
# device; this may be used with UUID= as a more robust way to name devices
# that works even if disks are added and removed. See fstab(5).
#
# <file system> <mount point>   <type>  <options>       <dump>  <pass>
PARTUUID=$ROOTPARTUUID  /               f2fs    defaults        0       1
PARTUUID=$BOOTPARTUUID  /boot           ext4    defaults        0       1

EOF

if [[ $? != 0 ]] ; then echo Extracting $IMAGE failed.; umount ${PART} ; exit 1; fi


echo "Syncing"
sync
umount rootfs/boot
umount rootfs
if [[ -n "$DISKIMAGE" ]] ; then losetup -d $DISK ; fi
rm rootfs -rf
echo "Done"
